# Generic motion controller definition
# All controller plugins inherit features from this one

from __future__ import absolute_import
from __future__ import print_function

class _ControllerGeneric:
	def test(self):
		print("test supergen")




#!/usr/bin/python
# -*- coding: ascii -*-

# Author: @harvie Tomas Mudrunka
# Date: 7 july 2018

from __future__ import absolute_import
from __future__ import print_function
__author__ = "@harvie Tomas Mudrunka"
#__email__  = ""

__name__ = _("Intersection")
__version__ = "0.0.1"

import math
import os.path
import re
from CNC import CNC,Block
from ToolsPage import Plugin
from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod
from bpath import EPS,eq,Path, Segment
from copy import deepcopy

class Tool(Plugin):
	__doc__ = _("""Intersection of two shapes""")			#<<< This comment will be show as tooltip for the ribbon button
	def __init__(self, master):
		Plugin.__init__(self, master,"Intersection")
		#Helical_Descent: is the name of the plugin show in the tool ribbon button
		self.icon = "intersection"			#<<< This is the name of png file used as icon for the ribbon button. It will be search in the "icons" subfolder
		self.group = "Development"	#<<< This is the name of group that plugin belongs
		self.oneshot = True
		#Here we are creating the widgets presented to the user inside the plugin
		#Name, Type , Default value, Description
		self.variables = [			#<<< Define a list of components for the GUI
			("name"    ,    "db" ,    "", _("Name")),							#used to store plugin settings in the internal database
		]
		self.buttons.append("exe")  #<<< This is the button added at bottom to call the execute method below

	# ----------------------------------------------------------------------
	# This method is executed when user presses the plugin execute button
	# ----------------------------------------------------------------------
	def execute(self, app):
		#print("go!")
		blocks  = []

		bid = app.editor.getSelectedBlocks()[0]
		xbasepath = app.gcode.toPath(bid)[0]

		bid = app.editor.getSelectedBlocks()[1]
		xislandpath = app.gcode.toPath(bid)[0]

		xbasepath.intersectPath(xislandpath)
		xislandpath.intersectPath(xbasepath)

		#xnewisland = self.pathBoolIntersection(xbasepath, xislandpath)
		xnewisland = self.pathBoolIntersection(xislandpath, xbasepath)

		#pth = Path("temp")
		#basepath.invert()
		#pth.extend(basepath)
		#pth.extend(basepath)
		##pth.invert()


		block = Block("intersect")
		block.extend(app.gcode.fromPath(xnewisland))
		blocks.append(block)

		#block = Block("diff")
		#block.extend(app.gcode.fromPath(pth))
		#blocks.append(block)


		active = app.activeBlock()
		app.gcode.insBlocks(active, blocks, "Intersect") #<<< insert blocks over active block in the editor
		app.refresh()                                                                                           #<<< refresh editor
		app.setStatus(_("Generated: Intersect"))                           #<<< feed back result
		#app.gcode.blocks.append(block)

##############################################


	def pol2car(self, r, phi, a=[0,0]):
		return [round(a[0]+r*cos(phi),4),round(a[1]+r*sin(phi),4)]

	def findSegment(self, path,A,B): #FIXME: not used for now...
		for seg in path:
			if seg.A == A and seg.B == B:
				return seg
			elif seg.A == B and seg.B == A:
				seg.invert()
				return seg
			else: return Segment(1, A, B)

	def findSubpath(self, path,A,B):
		path = deepcopy(path)
		newpath = self._findSubpath(path,A,B)
		if newpath is None:
			path.invert()
			newpath = self._findSubpath(path,A,B)
		return newpath

	def _findSubpath(self, path,A,B):
		print("finding", A, B)

		sub = None
		for i in xrange(0,len(path)*2): #iterate twice with wrap around
			j = i%len(path)
			seg = path[j]

			if eq(seg.A,A): sub = Path("subp")
			print("seg", sub is None, seg)
			if sub is not None: sub.append(seg)
			if eq(seg.B,B): break

		print("found", sub)
		return sub

	def pathBoolIntersection(self, basepath, islandpath):
		#basepath = deepcopy(basepath)
		#islandpath = deepcopy(islandpath)

		#find first intersecting segment
		first = None
		for i,segment in enumerate(islandpath):
			if basepath.isInside(segment.midPoint()): first = i
		if first is None:
			print("not intersecting paths")
			return None

		#generate intersected path
		newisland = Path("new")
		A = None
		for i in xrange(first,2*len(islandpath)+first):
			j = i%len(islandpath)
			segment = islandpath[j]
			if segment.length()<EPS: continue #ignore zero length segments
			if not basepath.isInside(segment.midPoint()):
				if A is None:
					A = segment.A
			else:
				if A is not None:
					newisland.extend(self.findSubpath(basepath,A,segment.A))
					print("new",newisland)
					A = None
				newisland.append(segment)
		#for i,seg in enumerate(newisland):
		#	newisland[i].correct();
		print("new2",newisland)
		return newisland



# -*- coding: ascii -*-
#
# Author: vvlachoudis@gmail.com
# Date: 18-Jun-2015

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Vasilis Vlachoudis"
__email__  = "vvlachoudis@gmail.com"

try:
	from Tkinter import *
except ImportError:
	from tkinter import *

import Utils
import tkExtra

_TABFONT    = ("Sans","-14","bold")
_FONT       = ("Sans","-11")

#_BACKGROUND_DISABLE = "#D6D2D0"
#_BACKGROUND         = "White"
#_BACKGROUND_GROUP   = "LightGray"
#_BACKGROUND_GROUP2  = "#D0E0E0"
#_FOREGROUND_GROUP   = "Black"

_BACKGROUND_DISABLE = "#A6A2A0"
_BACKGROUND         = "#E6E2E0"
_BACKGROUND_GROUP   = "#B6B2B0"

_BACKGROUND_GROUP2  = "#B0C0C0"
_BACKGROUND_GROUP3  = "#A0C0A0"
_BACKGROUND_GROUP4  = "#B0C0A0"

_FOREGROUND_GROUP   = "White"
_ACTIVE_COLOR       = "LightYellow"
_LABEL_SELECT_COLOR = "#C0FFC0"

# Ribbon show state
RIBBON_HIDDEN =  0	# Hidden
RIBBON_SHOWN  =  1	# Displayed
RIBBON_TEMP   = -1	# Show temporarily

#===============================================================================
# Frame Group with a button at bottom
#===============================================================================
class LabelGroup(Frame):
	def __init__(self, master, name, command=None, **kw):
		Frame.__init__(self, master, **kw)
		self.name = name
		self.config(	#bg="Green",
				background=_BACKGROUND,
				borderwidth=0,
				highlightthickness=0,
				pady=0)

		# right frame as a separator
		f = Frame(self, borderwidth=2, relief=GROOVE, background=_BACKGROUND_DISABLE)
		f.pack(side=RIGHT, fill=Y, padx=0, pady=0)

		# frame to insert the buttons
		self.frame = Frame(self,
				#bg="Orange",
				background=_BACKGROUND,
				padx=0,
				pady=0)
		self.frame.pack(side=TOP, expand=TRUE, fill=BOTH, padx=0, pady=0)

		if command:
			self.label = LabelButton(self, self, "<<%s>>"%(name), text=name)
			self.label.config(command=command,
				image=Utils.icons["triangle_down"],
				foreground=_FOREGROUND_GROUP,
				background=_BACKGROUND_GROUP,
				highlightthickness=0,
				borderwidth=0,
				pady=0,
				compound=RIGHT)
		else:
			self.label = Label(self, text=_(name),
					font       = _FONT,
					foreground = _FOREGROUND_GROUP,
					background = _BACKGROUND_GROUP,
					padx=2,
					pady=0)	# Button takes 1px for border width
		self.label.pack(side=BOTTOM, fill=X, pady=0)

	#-----------------------------------------------------------------------
	def grid2rows(self):
		self.frame.grid_rowconfigure(0, weight=1)
		self.frame.grid_rowconfigure(1, weight=1)

	#-----------------------------------------------------------------------
	def grid3rows(self):
		self.grid2rows()
		self.frame.grid_rowconfigure(2, weight=1)

#===============================================================================
class _KeyboardFocus:
	#-----------------------------------------------------------------------
	def _bind(self):
		self.bind("<Return>",		self._invoke)
		self.bind("<FocusIn>",		self._focusIn)
		self.bind("<FocusOut>",		self._focusOut)

	#-----------------------------------------------------------------------
	def _focusIn(self, event):
		self.__backgroundColor = self.cget("background")
		self.config(background = _ACTIVE_COLOR)

	#-----------------------------------------------------------------------
	def _focusOut(self, event):
		self.config(background = self.__backgroundColor)

	#-----------------------------------------------------------------------
	def _invoke(self, event):
		self.invoke()

#===============================================================================
# Button with Label that generates a Virtual Event or calls a command
#===============================================================================
class LabelButton(Button, _KeyboardFocus):
	def __init__(self, master, recipient=None, event=None, **kw):
		Button.__init__(self, master, **kw)
		self.config(	relief           = FLAT,
				activebackground = _ACTIVE_COLOR,
				font             = _FONT,
				borderwidth      = 1,
				highlightthickness = 0,
				padx             = 2,
				pady             = 0)
		_KeyboardFocus._bind(self)
		if recipient is not None:
			self.config(command = self.sendEvent)
			self._recipient = recipient
			self._event     = event
		else:
			self._recipient = None
			self._event     = None

	#-----------------------------------------------------------------------
	def sendEvent(self):
		self._recipient.event_generate(self._event)

#===============================================================================
class LabelCheckbutton(Checkbutton, _KeyboardFocus):
	def __init__(self, master, **kw):
		Checkbutton.__init__(self, master, **kw)
		self.config(	selectcolor        = _LABEL_SELECT_COLOR,
				activebackground   = _ACTIVE_COLOR,
				background         = _BACKGROUND,
				indicatoron        = FALSE,
				relief             = FLAT,
				borderwidth        = 0,
				highlightthickness = 0,
				padx               = 0,
				pady               = 0,
				font               = _FONT
			)
		_KeyboardFocus._bind(self)

#===============================================================================
class LabelRadiobutton(Radiobutton, _KeyboardFocus):
	def __init__(self, master, **kw):
		Radiobutton.__init__(self, master, **kw)
		self.config(
			selectcolor        = _LABEL_SELECT_COLOR,
			activebackground   = _ACTIVE_COLOR,
			background         = _BACKGROUND,
			indicatoron        = FALSE,
			borderwidth        = 0,
			highlightthickness = 0,
			pady               = 0,
			font               = _FONT
		)
		_KeyboardFocus._bind(self)

#===============================================================================
class LabelCombobox(tkExtra.Combobox, _KeyboardFocus):
	def __init__(self, master, **kw):
		tkExtra.Combobox.__init__(self, master, **kw)
		self.config(background=_BACKGROUND, font=_FONT)
		Frame.config(self, background=_BACKGROUND, padx=0, pady=0)
		_KeyboardFocus._bind(self)

	#-----------------------------------------------------------------------
	def _focusOut(self, event):
		self.config(background = _BACKGROUND) #self.__backgroundColor)
		Frame.config(self, background= _BACKGROUND) #self.__backgroundColor)

#===============================================================================
# Button with Label that popup a menu
#===============================================================================
class MenuButton(Button, _KeyboardFocus):
	def __init__(self, master, menulist, **kw):
		Button.__init__(self, master, **kw)
		self.config(	relief           = FLAT,
				activebackground = _ACTIVE_COLOR,
				font             = _FONT,
				borderwidth      = 0,
				highlightthickness= 0,
				padx             = 2,
				pady             = 0,
				command          = self.showMenu)

		_KeyboardFocus._bind(self)
		self.bind("<Return>", self.showMenu)
		if menulist is not None:
			self._menu = MenuButton.createMenuFromList(self, menulist)
		else:
			self._menu = None

	#-----------------------------------------------------------------------
	def showMenu(self, event=None):
		if self._menu is not None:
			self._showMenu(self._menu)
		else:
			self._showMenu(self.createMenu())

	#-----------------------------------------------------------------------
	def _showMenu(self, menu):
		if menu is not None:
			menu.tk_popup(
				self.winfo_rootx(),
				self.winfo_rooty() + self.winfo_height())

	#-----------------------------------------------------------------------
	def createMenu(self):
		return None

	#-----------------------------------------------------------------------
	@staticmethod
	def createMenuFromList(master, menulist):
		mainmenu = menu = Menu(master, tearoff=0, activebackground=_ACTIVE_COLOR)
		for item in menulist:
			if item is None:
				menu.add_separator()
			elif isinstance(item,str):
				menu = Menu(mainmenu)
				mainmenu.add_cascade(label=item, menu=menu)
			else:
				name, icon, cmd = item
				if icon is None: icon = "empty"
				menu.add_command(label=name,
						image=Utils.icons[icon],
						compound=LEFT,
						command=cmd)
		return menu

#===============================================================================
# A label group with a drop down menu
#===============================================================================
class MenuGroup(LabelGroup):
	def __init__(self, master, name, menulist=None, **kw):
		LabelGroup.__init__(self, master, name, command=self._showMenu, **kw)
		self._menulist = menulist

	#-----------------------------------------------------------------------
	def createMenu(self):
		if self._menulist is not None:
			return MenuButton.createMenuFromList(self, self._menulist)
		else:
			return None

	#-----------------------------------------------------------------------
	def _showMenu(self):
		menu = self.createMenu()
		if menu is not None:
			menu.tk_popup(
				self.winfo_rootx(),
				self.winfo_rooty() + self.winfo_height())

#===============================================================================
# Context group for a specific item in the Ribbon
#===============================================================================
#class ContextGroup(LabelGroup):
#	def __init__(self, master, name, **kw):
#		LabelGroup.__init__(self, master, name, **kw)

#===============================================================================
# Page Tab buttons
#===============================================================================
class TabButton(Radiobutton):
	def __init__(self, master, **kw):
		Radiobutton.__init__(self, master, **kw)
		self.config(	selectcolor        = _BACKGROUND,
				activebackground   = _ACTIVE_COLOR,
				indicatoron        = FALSE,
				relief             = FLAT,
				font               = _TABFONT,
				borderwidth        = 0,
				highlightthickness = 0,
				padx               = 5,
				pady               = 0,
				background         = _BACKGROUND_DISABLE
			)
		self.bind("<FocusIn>",		self._focusIn)
		self.bind("<FocusOut>",		self._focusOut)

	#-----------------------------------------------------------------------
	# Bind events on TabFrame
	#----------------------------------------------------------------------
	def bindClicks(self, tabframe):
		self.bind("<Double-1>",         tabframe.double)
		self.bind("<Button-1>",         tabframe.dragStart)
		self.bind("<B1-Motion>",        tabframe.drag)
		self.bind("<ButtonRelease-1>",  tabframe.dragStop)
		self.bind("<Control-ButtonRelease-1>", tabframe.pinActive)

		self.bind("<Left>",		tabframe._tabLeft)
		self.bind("<Right>",		tabframe._tabRight)
		self.bind("<Down>",		tabframe._tabDown)

	#----------------------------------------------------------------------
	def _focusIn(self, evenl=None):
		self.config(selectcolor = _ACTIVE_COLOR)

	#----------------------------------------------------------------------
	def _focusOut(self, evenl=None):
		self.config(selectcolor = _BACKGROUND)

#===============================================================================
# Page
#===============================================================================
class Page:		# <--- should be possible to be a toplevel as well
	_motionClasses = (LabelButton, LabelRadiobutton, LabelCheckbutton, LabelCombobox, MenuButton)
	_name_ = None
	_icon_ = None
	_doc_  = "Tooltip"

	#-----------------------------------------------------------------------
	def __init__(self, master, **kw):
		self.master = master
		self.name   = self._name_
		self._icon  = Utils.icons[self._icon_]
		self._tab   = None	# Tab button
		self.ribbons = []
		self.frames  = []
		self.init()
		self.create()

	#-----------------------------------------------------------------------
	# Override initialization
	#-----------------------------------------------------------------------
	def init(self):
		pass

	#-----------------------------------------------------------------------
	# The tab page can change master if undocked
	#-----------------------------------------------------------------------
	# FIXME XXX SHOULD BE REMOVED
	#-----------------------------------------------------------------------
	def create(self):
		self.createPage()
#		self.ribbonBindMotion()
#		self.refresh()

	#-----------------------------------------------------------------------
	# FIXME XXX SHOULD BE REMOVED
	#-----------------------------------------------------------------------
	def createPage(self):
		self.page = Frame(self.master._pageFrame)
		return self.page

	#-----------------------------------------------------------------------
	# Called when a page is activated
	#-----------------------------------------------------------------------
	def activate(self):
		pass

	#-----------------------------------------------------------------------
	def refresh(self):
		pass

	# ----------------------------------------------------------------------
	def canUndo(self):	return True
	def canRedo(self):	return True
	def resetUndo(self):	pass
	def undo(self, event=None): pass
	def redo(self, event=None): pass

	# ----------------------------------------------------------------------
	def refreshUndoButton(self):
		# Check if frame provides undo/redo
		if self.master is None: return
		if self.page is None: return

		if self.canUndo():
			state = NORMAL
		else:
			state = DISABLED
		self.master.tool["undo"].config(state=state)
		self.master.tool["undolist"].config(state=state)

		if self.canRedo():
			state = NORMAL
		else:
			state = DISABLED
		self.master.tool["redo"].config(state=state)

	#-----------------------------------------------------------------------
	def keyboardFocus(self):
		self._tab.focus_set()

	#-----------------------------------------------------------------------
	# Return the closest widget along a direction
	#-----------------------------------------------------------------------
	@staticmethod
	def __compareDown(x,y,xw,yw):  return yw>y+1
	@staticmethod
	def __compareUp(x,y,xw,yw):    return yw<y-1
	@staticmethod
	def __compareRight(x,y,xw,yw): return xw>x+1
	@staticmethod
	def __compareLeft(x,y,xw,yw):  return xw<x-1

	#-----------------------------------------------------------------------
	@staticmethod
	def __closest(widget, compare, x, y):
		closest = None
		dc2 = 10000000
		if widget is None: return closest, dc2
		for child in widget.winfo_children():
			for class_ in Page._motionClasses:
				if isinstance(child, class_):
					if child["state"] == DISABLED: continue
					xw = child.winfo_rootx()
					yw = child.winfo_rooty()
					if compare(x,y,xw,yw):
						d2 = (xw-x)**2 + (yw-y)**2
						if d2 < dc2:
							closest = child
							dc2 = d2
					break
			else:
				c,d2 = Page.__closest(child, compare, x, y)
				if d2 < dc2:
					closest = c
					dc2 = d2
		return closest, dc2

	#-----------------------------------------------------------------------
	# Select/Focus the closest element
	#-----------------------------------------------------------------------
	def _ribbonUp(self, event=None):
		x = event.widget.winfo_rootx()
		y = event.widget.winfo_rooty()
		closest,d2 = Page.__closest(self.ribbon, Page.__compareUp, x, y)
		if closest is not None:
			closest.focus_set()

	#-----------------------------------------------------------------------
	def _ribbonDown(self, event=None):
		x = event.widget.winfo_rootx()
		y = event.widget.winfo_rooty()
		closest,d2 = Page.__closest(self.ribbon, Page.__compareDown, x, y)
		if closest is not None:
			closest.focus_set()

	#-----------------------------------------------------------------------
	def _ribbonLeft(self, event=None):
		x = event.widget.winfo_rootx()
		y = event.widget.winfo_rooty()
		closest,d2 = Page.__closest(self.ribbon, Page.__compareLeft, x, y)
		if closest is not None:
			closest.focus_set()

	#-----------------------------------------------------------------------
	def _ribbonRight(self, event=None):
		x = event.widget.winfo_rootx()
		y = event.widget.winfo_rooty()
		closest,d2 = Page.__closest(self.ribbon, Page.__compareRight, x, y)
		if closest is not None:
			closest.focus_set()

#===============================================================================
# TabRibbonFrame
#===============================================================================
class TabRibbonFrame(Frame):
	def __init__(self, master, **kw):
		Frame.__init__(self, master, kw)
		self.config(background=_BACKGROUND_DISABLE)

		self.oldActive  = None
		self.activePage = StringVar(self)
		self.tool       = {}
		self.pages      = {}

		# === Top frame with buttons ===
		frame = Frame(self, background=_BACKGROUND_DISABLE)
		frame.pack(side=TOP, fill=X)

		# --- Basic buttons ---
		b = LabelButton(frame, self, "<<New>>",
				image=Utils.icons["new"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("New file"))
		b.pack(side=LEFT)

		b = LabelButton(frame, self, "<<Open>>",
				image=Utils.icons["load"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("Open file [Ctrl-O]"))
		b.pack(side=LEFT)

		b = LabelButton(frame, self, "<<Save>>",
				image=Utils.icons["save"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("Save all [Ctrl-S]"))
		b.pack(side=LEFT)

		b = LabelButton(frame, self, "<<Undo>>",
				image=Utils.icons["undo"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("Undo [Ctrl-Z]"))
		b.pack(side=LEFT)
		self.tool["undo"] = b

		b = LabelButton(frame, image=Utils.icons["triangle_down"],
				command=self.undolist,
				background=_BACKGROUND_DISABLE)
		b.pack(side=LEFT)
		self.tool["undolist"] = b

		b = LabelButton(frame, self, "<<Redo>>",
				image=Utils.icons["redo"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("Redo [Ctrl-Y]"))
		b.pack(side=LEFT)
		self.tool["redo"] = b

		Label(frame, image=Utils.icons["sep"],
				background=_BACKGROUND_DISABLE).pack(side=LEFT, padx=3)

		# --- Help ---
		b = LabelButton(frame, self, "<<Help>>",
				image=Utils.icons["info"],
				background=_BACKGROUND_DISABLE)
		tkExtra.Balloon.set(b, _("Help [F1]"))
		b.pack(side=RIGHT, padx=2)

		Label(frame, image=Utils.icons["sep"],
				background=_BACKGROUND_DISABLE).pack(side=RIGHT, padx=3)

		# --- TabBar ---
		self._tabFrame = Frame(frame, background=_BACKGROUND_DISABLE)
		self._tabFrame.pack(side=LEFT, fill=BOTH, expand=YES)

		# ==== Ribbon Frame ====
		self._ribbonFrame = Frame(self,
						background=_BACKGROUND,
						pady=0,
						relief=RAISED)
		self._ribbonFrame.pack(fill=BOTH, expand=YES, padx=0, pady=0)

		self.setPageFrame(None)

	#-----------------------------------------------------------------------
	def setPageFrame(self, frame):
		self._pageFrame = frame

	#-----------------------------------------------------------------------
	def undolist(self, event=None): self.event_generate("<<UndoList>>")

	#-----------------------------------------------------------------------
	def getActivePage(self):
		return self.pages[self.activePage.get()]

	#-----------------------------------------------------------------------
	# Add page to the tabs
	#-----------------------------------------------------------------------
	def addPage(self, page, side=LEFT):
		self.pages[page.name] = page
		page._tab = TabButton(self._tabFrame,
				image    = page._icon,
				text     = _(page.name),
				compound = LEFT,
				value    = page.name,
				variable = self.activePage,
				command  = self.changePage)
		tkExtra.Balloon.set(page._tab, page.__doc__)

		page._tab.pack(side=side, fill=Y, padx=5)

	# ----------------------------------------------------------------------
	# Unpack the old page
	# ----------------------------------------------------------------------
	def _forgetPage(self):
		if self.oldActive:
			for frame,args in self.oldActive.ribbons:
				frame.pack_forget()
			for frame,args in self.oldActive.frames:
				frame.pack_forget()
			self.oldActive = None

	# ----------------------------------------------------------------------
	# Change ribbon and page
	# ----------------------------------------------------------------------
	def changePage(self, page=None):
		#import traceback
		#traceback.print_stack()

		if page is not None:
			if not isinstance(page, Page):
				try:
					page = self.pages[page]
				except KeyError:
					return
			self.activePage.set(page.name)
		else:
			try:
				page = self.pages[self.activePage.get()]
			except KeyError:
				return

		if page is self.oldActive: return

		self._forgetPage()

		for frame,args in page.ribbons:
			frame.pack(in_=self._ribbonFrame, **args)

		for frame,args in page.frames:
			frame.pack(in_=self._pageFrame, **args)

		self.oldActive = page
		page.activate()
		self.event_generate("<<ChangePage>>", data=page.name)

#	#-----------------------------------------------------------------------
#	# Give focus to the tab on the left
#	#-----------------------------------------------------------------------
#	def _tabLeft(self, event=None):
#		slaves = self._tabFrame.pack_slaves()
#		try:
#			pos = slaves.index(event.widget)-1
#		except ValueError:
#			if event.widget is self.dynamic:
#				pos = len(slaves)-1
#			else:
#				return
#		if pos < 0: return	# Do not replace First tab
#		slaves[pos].select()
#		#self.changePage()
#		slaves[pos].focus_set()
#
#	#-----------------------------------------------------------------------
#	# Give focus to the tab on the right
#	#-----------------------------------------------------------------------
#	def _tabRight(self, event=None):
#		slaves = self._tabFrame.pack_slaves()
#		try:
#			pos = slaves.index(event.widget)+1
#		except ValueError:
#			return
#		if pos < len(slaves):
#			slaves[pos].select()
#			#self.changePage()
#			slaves[pos].focus_set()
#		else:
#			# Open dynamic menu
#			self.dynamic.select()
#			self.dynamic.focus_set()
#			self.dynamicMenu()
#



#!/usr/bin/python
# -*- coding: latin1 -*-
#
# Author: vvlachoudis@gmail.com
# Date: 20-Oct-2015

from __future__ import absolute_import
import sys
import bmath

#===============================================================================
# Cardinal cubic spline class
#===============================================================================
class CardinalSpline:
	def __init__(self, A=0.5):
		# The default matrix is the Catmull-Rom splin
		# which is equal to Cardinal matrix
		# for A = 0.5
		#
		# Note: Vasilis
		#	The A parameter should be the fraction in t where
		#	the second derivative is zero
		self.setMatrix(A)

	#-----------------------------------------------------------------------
	# Set the matrix according to Cardinal
	#-----------------------------------------------------------------------
	def setMatrix(self, A=0.5):
		self.M = []
		self.M.append([  -A,  2.-A,    A-2.,   A ])
		self.M.append([2.*A,  A-3., 3.-2.*A,  -A ])
		self.M.append([  -A,    0.,       A,   0.])
		self.M.append([  0.,    1.,       0,   0.])

	#-----------------------------------------------------------------------
	# Evaluate Cardinal spline at position t
	# @param P	  list or tuple with 4 points y positions
	# @param t [0..1] fraction of interval from points 1..2
	# @param k	  index of starting 4 elements in P
	# @return spline evaluation
	#-----------------------------------------------------------------------
	def __call__(self, P, t, k=1):
		T = [t*t*t, t*t, t, 1.0]
		R = [0.0]*4
		for i in range(4):
			for j in range(4):
				R[i] += T[j] * self.M[j][i]
		y = 0.0
		for i in range(4):
			y += R[i]*P[k+i-1]

		return y

	#-----------------------------------------------------------------------
	# Return the coefficients of a 3rd degree polynomial
	#     f(x) = a t^3 + b t^2 + c t + d
	# @return [a, b, c, d]
	#-----------------------------------------------------------------------
	def coefficients(self, P, k=1):
		C = [0.0]*4
		for i in range(4):
			for j in range(4):
				C[i] += self.M[i][j] * P[k+j-1]
		return C

	#-----------------------------------------------------------------------
	# Evaluate the value of the spline using the coefficients
	#-----------------------------------------------------------------------
	def evaluate(self, C, t):
		return ((C[0]*t + C[1])*t + C[2])*t + C[3]

#===============================================================================
# Cubic spline ensuring that the first and second derivative are continuous
# adapted from Penelope Manual Appending B.1
# It requires all the points (xi,yi) and the assumption on how to deal
# with the second derviative on the extremeties
# Option 1: assume zero as second derivative on both ends
# Option 2: assume the same as the next or previous one
#===============================================================================
class CubicSpline:
	def __init__(self, X, Y):
		self.X = X
		self.Y = Y
		self.n = len(X)

		# Option #1
		s1 = 0.0	# zero based = s0
		sN = 0.0	# zero based = sN-1

		# Construct the tri-diagonal matrix
		A = []
		B = [0.0] * (self.n-2)
		for i in range(self.n-2):
			A.append([0.0] * (self.n-2))

		for i in range(1,self.n-1):
			hi = self.h(i)
			Hi = 2.0*(self.h(i-1) + hi)
			j = i-1
			A[j][j] = Hi
			if i+1<self.n-1:
				A[j][j+1] = A[j+1][j] = hi

			if i==1:
				B[j] = 6.*(self.d(i) - self.d(j)) - hi*s1
			elif i<self.n-2:
				B[j] = 6.*(self.d(i) - self.d(j))
			else:
				B[j] = 6.*(self.d(i) - self.d(j)) - hi*sN

#		from pprint import pprint
#		print "=========== A ============="
#		pprint(A)
#		print "=========== B ============="
#		pprint(B)

		# Solve by gauss elimination
#		AA = bmath.Matrix(A)
#		BB = []
#		for b in B: BB.append([b])
#		BB = bmath.Matrix(BB)
#		print AA
#		print BB
#		AA.inv()
#		print AA*BB
		self.s = bmath.gauss(A,B)
		self.s.insert(0,s1)
		self.s.append(sN)
#		print ">> s <<"
#		pprint(self.s)

	#-----------------------------------------------------------------------
	def h(self, i):
		return self.X[i+1] - self.X[i]

	#-----------------------------------------------------------------------
	def d(self, i):
		return (self.Y[i+1] - self.Y[i]) / (self.X[i+1] - self.X[i])

	#-----------------------------------------------------------------------
	def coefficients(self, i):
		"""return coefficients of cubic spline for interval i a*x**3+b*x**2+c*x+d"""
		hi  = self.h(i)
		si  = self.s[i]
		si1 = self.s[i+1]
		xi  = self.X[i]
		xi1 = self.X[i+1]
		fi  = self.Y[i]
		fi1 = self.Y[i+1]

		a = 1./(6.*hi)*(si*xi1**3 - si1*xi**3 + 6.*(fi*xi1 - fi1*xi)) + hi/6.*(si1*xi - si*xi1)
		b = 1./(2.*hi)*(si1*xi**2 - si*xi1**2 + 2*(fi1 - fi)) + hi/6.*(si - si1)
		c = 1./(2.*hi)*(si*xi1 - si1*xi)
		d = 1./(6.*hi)*(si1-si)

		return [d,c,b,a]

	#-----------------------------------------------------------------------
	def __call__(self, i, x):
		# FIXME should interpolate to find the interval
		C = self.coefficients(i)
		return ((C[0]*x + C[1])*x + C[2])*x + C[3]

	#-----------------------------------------------------------------------
	# @return evaluation of cubic spline at x using coefficients C
	#-----------------------------------------------------------------------
	def evaluate(self, C, x):
		return ((C[0]*x + C[1])*x + C[2])*x + C[3]

	#-----------------------------------------------------------------------
	# Return evaluated derivative at x using coefficients C
	#-----------------------------------------------------------------------
	def derivative(self, C, x):
		a = 3.0*C[0]			# derivative coefficients
		b = 2.0*C[1]			# ... for sampling with rejection
		c =     C[2]
		return (3.0*C[0]*x + 2.0*C[1])*x + C[2]

# ------------------------------------------------------------------------------
# Convert a B-spline to polyline with a fixed number of segments
#
# FIXME to become adaptive
# ------------------------------------------------------------------------------
def spline2Polyline(xyz, degree, closed, segments, knots):
	# Check if last point coincide with the first one
	if (bmath.Vector(xyz[0])-bmath.Vector(xyz[-1])).length2() < 1e-10:
		# it is already closed, treat it as open
		closed = False
		# FIXME we should verify if it is periodic,.... but...
		#       I am not sure :)

	if closed:
		xyz.extend(xyz[:degree])
		knots = None
	else:
		# make base-1
		knots.insert(0, 0)

	npts = len(xyz)

	if degree<1 or degree>3:
		#print "invalid degree"
		return None,None,None

	# order:
	k = degree+1

	if npts < k:
		#print "not enough control points"
		return None,None,None

	# resolution:
	nseg = segments * npts

	# WARNING: base 1
	b = [0.0]*(npts*3+1)		# polygon points
	h = [1.0]*(npts+1)		# set all homogeneous weighting factors to 1.0
	p = [0.0]*(nseg*3+1)		# returned curved points

	i = 1
	for pt in xyz:
		b[i]   = pt[0]
		b[i+1] = pt[1]
		b[i+2] = pt[2]
		i +=3

	#if periodic:
	if closed:
		_rbsplinu(npts, k, nseg, b, h, p, knots)
	else:
		_rbspline(npts, k, nseg, b, h, p, knots)

	x = []
	y = []
	z = []
	for i in range(1,3*nseg+1,3):
		x.append(p[i])
		y.append(p[i+1])
		z.append(p[i+2])

#	for i,xyz in enumerate(zip(x,y,z)):
#		print i,xyz

	return x,y,z

# ------------------------------------------------------------------------------
# Subroutine to generate a B-spline open knot vector with multiplicity
# equal to the order at the ends.
#    c            = order of the basis function
#    n            = the number of defining polygon vertices
#    n+2          = index of x[] for the first occurence of the maximum knot vector value
#    n+order      = maximum value of the knot vector -- $n + c$
#    x[]          = array containing the knot vector
# ------------------------------------------------------------------------------
def _knot(n, order):
	x = [0.0]*(n+order+1)
	for i in range(2, n+order+1):
		if i>order and i<n+2:
			x[i] = x[i-1] + 1.0
		else:
			x[i] = x[i-1]
	return x

# ------------------------------------------------------------------------------
# Subroutine to generate a B-spline uniform (periodic) knot vector.
#
# order        = order of the basis function
# n            = the number of defining polygon vertices
# n+order      = maximum value of the knot vector -- $n + order$
# x[]          = array containing the knot vector
# ------------------------------------------------------------------------------
def _knotu(n, order):
	x = [0]*(n+order+1)
	for i in range(2, n+order+1):
		x[i] = float(i-1)
	return x

# ------------------------------------------------------------------------------
# Subroutine to generate rational B-spline basis functions--open knot vector

# C code for An Introduction to NURBS
# by David F. Rogers. Copyright (C) 2000 David F. Rogers,
# All rights reserved.

# Name: rbasis
# Subroutines called: none
# Book reference: Chapter 4, Sec. 4. , p 296

#   c        = order of the B-spline basis function
#   d        = first term of the basis function recursion relation
#   e        = second term of the basis function recursion relation
#   h[]      = array containing the homogeneous weights
#   npts     = number of defining polygon vertices
#   nplusc   = constant -- npts + c -- maximum number of knot values
#   r[]      = array containing the rational basis functions
#              r[1] contains the basis function associated with B1 etc.
#   t        = parameter value
#   temp[]   = temporary array
#   x[]      = knot vector
# ------------------------------------------------------------------------------
def _rbasis(c, t, npts, x, h, r):
	nplusc = npts + c
	temp = [0.0]*(nplusc+1)

	# calculate the first order non-rational basis functions n[i]
	for i in range(1, nplusc):
		if x[i] <= t < x[i+1]:
			temp[i] = 1.0
		else:
			temp[i] = 0.0

	# calculate the higher order non-rational basis functions
	for k in range(2,c+1):
		for i in range(1,nplusc-k+1):
			# if the lower order basis function is zero skip the calculation
			if temp[i] != 0.0:
				d = ((t-x[i])*temp[i])/(x[i+k-1]-x[i])
			else:
				d = 0.0

			# if the lower order basis function is zero skip the calculation
			if temp[i+1] != 0.0:
				e = ((x[i+k]-t)*temp[i+1])/(x[i+k]-x[i+1])
			else:
				e = 0.0
			temp[i] = d + e

	# pick up last point
	if t >= x[nplusc]:
		temp[npts] = 1.0

	# calculate sum for denominator of rational basis functions
	s = 0.0
	for i in range(1,npts+1):
		s += temp[i]*h[i]

	# form rational basis functions and put in r vector
	for i in range(1, npts+1):
		if s != 0.0:
			r[i] = (temp[i]*h[i])/s
		else:
			r[i] = 0

# ------------------------------------------------------------------------------
# Generates a rational B-spline curve using a uniform open knot vector.
#
# C code for An Introduction to NURBS
# by David F. Rogers. Copyright (C) 2000 David F. Rogers,
# All rights reserved.
#
# Name: rbspline.c
# Subroutines called: _knot, rbasis
# Book reference: Chapter 4, Alg. p. 297
#
#    b           = array containing the defining polygon vertices
#                  b[1] contains the x-component of the vertex
#                  b[2] contains the y-component of the vertex
#                  b[3] contains the z-component of the vertex
#    h           = array containing the homogeneous weighting factors
#    k           = order of the B-spline basis function
#    nbasis      = array containing the basis functions for a single value of t
#    nplusc      = number of knot values
#    npts        = number of defining polygon vertices
#    p[,]        = array containing the curve points
#                  p[1] contains the x-component of the point
#                  p[2] contains the y-component of the point
#                  p[3] contains the z-component of the point
#    p1          = number of points to be calculated on the curve
#    t           = parameter value 0 <= t <= npts - k + 1
#    x[]         = array containing the knot vector
# ------------------------------------------------------------------------------
def _rbspline(npts, k, p1, b, h, p, x):
	nplusc = npts + k
	nbasis = [0.0]*(npts+1)		# zero and re-dimension the basis array

	# generate the uniform open knot vector
	if x is None or len(x) != nplusc+1:
		x = _knot(npts, k)
	icount = 0
	# calculate the points on the rational B-spline curve
	t = 0
	step = float(x[nplusc])/float(p1-1)
	for i1 in range(1, p1+1):
		if x[nplusc] - t < 5e-6:
			t = x[nplusc]
		# generate the basis function for this value of t
		nbasis = [0.0]*(npts+1)	# zero and re-dimension the knot vector and the basis array
		_rbasis(k, t, npts, x, h, nbasis)
		# generate a point on the curve
		for j in range(1, 4):
			jcount = j
			p[icount+j] = 0.0
			# Do local matrix multiplication
			for i in range(1, npts+1):
				p[icount+j] +=  nbasis[i]*b[jcount]
				jcount += 3
		icount += 3
		t += step

# ------------------------------------------------------------------------------
# Subroutine to generate a rational B-spline curve using an uniform periodic knot vector
#
# C code for An Introduction to NURBS
# by David F. Rogers. Copyright (C) 2000 David F. Rogers,
# All rights reserved.
#
# Name: rbsplinu.c
# Subroutines called: _knotu, _rbasis
# Book reference: Chapter 4, Alg. p. 298
#
#   b[]         = array containing the defining polygon vertices
#                 b[1] contains the x-component of the vertex
#                 b[2] contains the y-component of the vertex
#                 b[3] contains the z-component of the vertex
#   h[]         = array containing the homogeneous weighting factors
#   k           = order of the B-spline basis function
#   nbasis      = array containing the basis functions for a single value of t
#   nplusc      = number of knot values
#   npts        = number of defining polygon vertices
#   p[,]        = array containing the curve points
#                 p[1] contains the x-component of the point
#                 p[2] contains the y-component of the point
#                 p[3] contains the z-component of the point
#   p1          = number of points to be calculated on the curve
#   t           = parameter value 0 <= t <= npts - k + 1
#   x[]         = array containing the knot vector
# ------------------------------------------------------------------------------
def _rbsplinu(npts, k, p1, b, h, p, x=None):
	nplusc = npts + k
	nbasis = [0.0]*(npts+1)		# zero and re-dimension the basis array
	# generate the uniform periodic knot vector
	if x is None or len(x) != nplusc+1:
		# zero and re dimension the knot vector and the basis array
		x = _knotu(npts, k)
	icount = 0
	# calculate the points on the rational B-spline curve
	t = k-1
	step = (float(npts)-(k-1))/float(p1-1)
	for i1 in range(1, p1+1):
		if x[nplusc] - t < 5e-6:
			t = x[nplusc]
		# generate the basis function for this value of t
		nbasis = [0.0]*(npts+1)
		_rbasis(k, t, npts, x, h, nbasis)
		# generate a point on the curve
		for j in range(1,4):
			jcount = j
			p[icount+j] = 0.0
			#  Do local matrix multiplication
			for i in range(1,npts+1):
				p[icount+j] += nbasis[i]*b[jcount]
				jcount += 3
		icount += 3
		t += step

# =============================================================================
if __name__ == "__main__":
	SPLINE_SEGMENTS = 20
	from .dxf import DXF
#	from dxfwrite.algebra import CubicSpline, CubicBezierCurve
	dxf = DXF(sys.argv[1],"r")
	dxf.readFile()
	dxf.close()
	for name,layer in dxf.layers.items():
		for entity in layer.entities:
			if entity.type == "SPLINE":
				xy = zip(entity[10], entity[20])
				x,y = spline2Polyline(xy, int(entity[71]), True, SPLINE_SEGMENTS)
				#for a,b in zip(x,y):
				#	print a,b



__package_name__ = 'numpy-stl'
__import_name__ = 'stl'
__version__ = '2.7.0'
__author__ = 'Rick van Hattem'
__author_email__ = 'Wolph@Wol.ph'
__description__ = ' '.join('''
Library to make reading, writing and modifying both binary and ascii STL files
easy.
'''.split())
__url__ = 'https://github.com/WoLpH/numpy-stl/'




from __future__ import absolute_import
import math
import struct

ASCII_FACET = """facet normal {normal[0]:.4f} {normal[1]:.4f} {normal[2]:.4f}
outer loop
vertex {face[0][0]:.4f} {face[0][1]:.4f} {face[0][2]:.4f}
vertex {face[1][0]:.4f} {face[1][1]:.4f} {face[1][2]:.4f}
vertex {face[2][0]:.4f} {face[2][1]:.4f} {face[2][2]:.4f}
endloop
endfacet
"""

BINARY_HEADER ="80sI"
BINARY_FACET = "12fH"

def crossproduct(u,v):
	s1 = u[1]*v[2] - u[2]*v[1]
	s2 = u[2]*v[0] - u[0]*v[2]
	s3 = u[0]*v[1] - u[1]*v[0]
	return [s1, s2, s3]

def norm(v):
	return math.sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2])

def normalize(v):
	m = norm(v)
	return [v[0]/m, v[1]/m, v[2]/m]

def normalto(u,v):
	return normalize(crossproduct(u,v))

def diff(u,v):
	return [u[0]-v[0], u[1]-v[1], u[2]-v[2]]

#functions for calculation of the normal vector (right-thumb-rule)
def crossProduct(a, b):
	#calculate cross product of two threedimensional vectors
	if (len(a)!=3) or (len(b)!=3):
		raise ValueError('unvalid value')
	return [a[1]*b[2] - a[2]*b[1], a[2]*b[0] - a[0]*b[2], a[0]*b[1] - a[1]*b[0]]

def diff(v1,v2):
	#substracting one list from another
	if (len(v1)!=3) or (len(v2)!=3):
		raise ValueError('unvalid value')
	return [v1[0]-v2[0],v1[1]-v2[1],v1[2]-v2[2]]

def normal(v1,v2,v3):
	#calculate normal vector on triangle spanned by the three vertices
	#tells in which direction the plane looks "right-thumb-rule"
	if (len(v1)!=3) or (len(v2)!=3) or (len(v2)!=3):
		raise ValueError('unvalid value')
	n=crossProduct(diff(v2,v1),diff(v3,v1))
	absolut=0
	for i in n:
		if i>=0:
			absolut+=i
		else:
			absolut-=i
	if absolut == 0:
		#print "this should not have happened!"
		return n
	else :
		return [n[0]/absolut,n[1]/absolut,n[2]/absolut]

#simple function to create sets of three vertices
def triangulate(vertices):
	n=len(vertices)
	if(n==3):
		return vertices
	elif n < 3:
			facets = triangulate(facet)
			self.add_facets(facets)
	else:
		 raise ValueError('wrong number of vertices')

	def add_facets(self, facets):
		#print "add %d facets" % len(facets)
		for facet in facets:
			self.add_facet(facet)

	def extrude(self,bottom,height):
		if len(bottom) < 3 :
			raise ValueError('not a polygon')
		else :
			top = []

			for vertice in bottom :
				top.append([vertice[0],vertice[1],vertice[2]+height])

			bottom.reverse()
			self.add_facet(bottom)
			bottom.reverse()

			for i in range(0,len(bottom)-1) :
				self.add_facet([bottom[i],bottom[i+1],top[i+1],top[i]])
			self.add_facet([bottom[len(bottom)-1],bottom[0],top[0],top[len(bottom)-1]])

			self.add_facet(top)


class ASCII_STL_Writer:
	""" Export 3D objects build of 3 or 4 vertices as ASCII STL file.
	"""
	def __init__(self, stream):
		self.fp = stream
		self._write_header()

	def _write_header(self):
		self.fp.write("solid python\n")

	def close(self):
		self.fp.write("endsolid python\n")

	def _write(self, face):
		n = normalto(diff(face[0],face[1]),diff(face[1],face[2]))
		self.fp.write(ASCII_FACET.format(normal=n,face=face))

	def _split(self, face):
		p1, p2, p3, p4 = face
		return (p1, p2, p3), (p3, p4, p1)

	def add_face(self, face):
		""" Add one face with 3 or 4 vertices. """
		if len(face) == 4:
			face1, face2 = self._split(face)
			self._write(face1)
			self._write(face2)
		elif len(face) == 3:
			self._write(face)
		else:
			raise ValueError('only 3 or 4 vertices for each face')

	def add_faces(self, faces):
		""" Add many faces. """
		for face in faces:
			self.add_face(face)

class Binary_STL_Writer(ASCII_STL_Writer):
	""" Export 3D objects build of 3 or 4 vertices as binary STL file.
	"""
	def __init__(self, stream):
		self.counter = 0
		ASCII_STL_Writer.__init__(self,stream)

	def close(self):
		self._write_header()

	def _write_header(self):
		self.fp.seek(0)
		self.fp.write(struct.pack(BINARY_HEADER, b'Python Binary STL Writer', self.counter))

	def _write(self, face):
		self.counter += 1
		n = normalto(diff(face[0],face[1]),diff(face[1],face[2]))
		data = [
			n[0], n[1], n[2],
			face[0][0], face[0][1], face[0][2],
			face[1][0], face[1][1], face[1][2],
			face[2][0], face[2][1], face[2][2],
			0
		]
		self.fp.write(struct.pack(BINARY_FACET, *data))
